#include "csquaresprite.h"



// Initialize sprite.
CSquareSprite::CSquareSprite(int xTexture, int yTexture, int widthTexture, int heightTexture)
{
	m_xTexture = xTexture;
	m_yTexture = yTexture;
	m_widthTexture = widthTexture;
	m_heightTexture = heightTexture;
	m_xPos = 0;
	m_yPos = 0;
	m_xTexturePos = 0;
	m_yTexturePos = 0;
	m_isVisible = true;
	m_Red = 0x80;
	m_Green = 0x80;
	m_Blue = 0x80;
	m_Alpha = 0x80;

}

// Clean up.
CSquareSprite::~CSquareSprite()
{
}

// Set Position.
void CSquareSprite::setPos(int xPos, int yPos) {
	m_xPos = xPos;
	m_yPos = yPos;
}

// Move sprite x, y positions.
void CSquareSprite::movePos(int xPos, int yPos) {
	m_xPos += xPos;
	m_yPos += yPos;
}

// Which part of the texture to use.
void CSquareSprite::setTexturePos(int xTexturePos, int yTexturePos) {
	m_xTexturePos = xTexturePos;
	m_yTexturePos = yTexturePos;
}

// Set sprite visibility.
void CSquareSprite::setVisible(bool isVisible) {
	m_isVisible = isVisible;
}

// Set Sprite color
void CSquareSprite::setColor(int Red, int Green, int Blue, int Alpha) {
	m_Red = Red;
	m_Green = Green;
	m_Blue = Blue;
	m_Alpha = Alpha;
}

// Is the sprite visible ???
bool CSquareSprite::getVisible(void) {
	return m_isVisible;
}

// Get X position
int CSquareSprite::getXPos(void) {
	return m_xPos;
}

// Get Y position
int CSquareSprite::getYPos(void) {
	return m_yPos;
}

// Get Width
int CSquareSprite::getWidth(void) {
	return m_widthTexture;
}

// Get Height
int CSquareSprite::getHeight(void) {
	return m_heightTexture;
}

// Collision test, only a simple inside squares test.
bool CSquareSprite::inside(CSquareSprite inSprite) {

	// Dont test if not visible.
	if (!m_isVisible) {
		return false;
	}

	// Here we go...
    if (((m_xPos + m_widthTexture) < inSprite.getXPos()) || (m_xPos > (inSprite.getXPos() + inSprite.getWidth())) ||
    ((m_yPos + m_heightTexture) < inSprite.getYPos()) || (m_yPos > (inSprite.getYPos() + inSprite.getHeight()))) {

    return false;
    } else {
    return true;
	}
}


// Just add the sprite to the beginning of the allocated Memory.
void CSquareSprite::addSprite(char *& pMem, int Zvalue) {

		// Only render visible sprites.
		if (!m_isVisible) {
			return;
		}

		// Add a GIF Tag 
		sps2GIFTag_t * pGIFTag = (sps2GIFTag_t *) pMem;
 
	  	// Create the GIFTag  
  		pGIFTag->i128=0;       			// Blank it out.
	  	pGIFTag->s.NLOOP=1;			   	// How many sprites we are rendering.
  		pGIFTag->s.PRE=1;      			// Send our PRIM value to the GIF via this GIFTag.
	  	pGIFTag->s.EOP=1;      			// This is the end of the packets, no more GIFTags are to be expected after the sprites.
		pGIFTag->s.PRIM = GS_PRIM(PRIM_SPRITE, 
							PRIM_IIP_FLAT, 
							PRIM_TME_ON,
							PRIM_FGE_OFF, 
							PRIM_ABE_ON, 
							PRIM_AA1_OFF, 
							PRIM_FST_UV, 
							PRIM_CTXT_CONTEXT1, 
							PRIM_FIX_NOFIXDDA);
	  	pGIFTag->s.FLG=GIF_FLG_PACKED;	// Use packed format for data.
  		pGIFTag->s.NREG=0;     			// Start at 0 and increment as needed.
  
	  	// Add RGBAQ register.
  		SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_RGBAQ);
	  	// Add UV register.
  		SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_UV);
	  	// Add XYZ2 register.
  		SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_XYZ2);
	  	// Add UV register.
  		SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_UV);
	  	// Add XYZ2 register.
  		SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_XYZ2);
		pGIFTag++;

		// Update memory pointer.
		pMem = (char *)pGIFTag;

		addSpriteToGIFTag(pMem, Zvalue);
}


// This can be used with or without the addSprite function, slightly more efficent standalone but more code...
void CSquareSprite::addSpriteToGIFTag(char *& pMem, int Zvalue) {

	// Only render visible sprites.
	if (!m_isVisible) {
		return;
	}

	// pGIFReg is now pointing to the first memory position.
  	sps2GIFPackedRegister_t * pGIFReg = (sps2GIFPackedRegister_t *) pMem;

   	// Prepare the RGBAQ register
   	pGIFReg->i128=0;
   	pGIFReg->RGBAQ.s.R=m_Red;	// Red
   	pGIFReg->RGBAQ.s.G=m_Green;	// Green
   	pGIFReg->RGBAQ.s.B=m_Blue;	// Blue
   	pGIFReg->RGBAQ.s.A=m_Alpha;	// Alpha
	pGIFReg++;

   	// Prepare the first UV register
   	pGIFReg->i128=0;
   	pGIFReg->UV.s.U=(m_xTexture + (m_xTexturePos*m_widthTexture)) << 4;
   	pGIFReg->UV.s.V=(m_yTexture + (m_yTexturePos*m_heightTexture)) << 4;
	pGIFReg++;

   	// Prepare the first XYZ2 register
   	pGIFReg->i128=0;
   	pGIFReg->XYZ2.s.X=(m_xPos + 1728) << 4;
   	pGIFReg->XYZ2.s.Y=(m_yPos + 1808) << 4;
   	pGIFReg->XYZ2.s.Z=Zvalue;
	pGIFReg++;

   	// Prepare the second UV register
   	pGIFReg->i128=0;
   	pGIFReg->UV.s.U=(m_xTexture + m_widthTexture + (m_xTexturePos*m_widthTexture)) << 4;
   	pGIFReg->UV.s.V=(m_yTexture + m_heightTexture + (m_yTexturePos*m_heightTexture)) << 4;
	pGIFReg++;

   	// Prepare the second XYZ2 register
   	pGIFReg->i128=0;
   	pGIFReg->XYZ2.s.X=(m_xPos + m_widthTexture + 1728) << 4;
   	pGIFReg->XYZ2.s.Y=(m_yPos + m_heightTexture + 1808) << 4;
   	pGIFReg->XYZ2.s.Z=Zvalue;
	pGIFReg++;

	// Update memory pointer.
	pMem = (char *)pGIFReg;
}